#include <QtGui>

class Widget : public QWidget
{
public:
	Widget();
protected:
	void drawArrow(QRectF bounds, QPainter& painter) const;
	virtual void paintEvent(QPaintEvent* p) override;
};

Widget::Widget()
: QWidget(NULL)
{
}

void Widget::paintEvent(QPaintEvent* e)
{
	QPainter p(this);
	drawArrow(QRectF(10, 10, 7, 7), p);
}

inline int largestOddNumber(int v)
{
	return (v & 1) ? v : v - 1;
}

template <typename T>
inline T round(double x)
{
	return static_cast<T>(x >= 0 ? x + 0.5 : x - 0.5);
}

void Widget::drawArrow(QRectF bounds, QPainter& painter) const
{
	const QRectF rectangleF = bounds;
	const QPointF centerF = rectangleF.center();

	const QPoint center(round<int>(centerF.x()), round<int>(centerF.y()));
	const int width = largestOddNumber(round<int>(rectangleF.width()));
	const int height = largestOddNumber(round<int>(rectangleF.height()));

	const int tmpSide = qMax(width, height);
	const int halfSide = tmpSide / 2;
	const int quarterSide = tmpSide / 4;

	const double xCenter = center.x() + 0.5;
	const double yCenter = center.y() + 0.5;
	const double left = xCenter - halfSide;
	const double right = xCenter + halfSide;
	const double top = yCenter - halfSide;
	const double bottom = yCenter + halfSide;

	const QPointF bottomPoint(xCenter, bottom);
	const QPointF leftPoint(left, yCenter);
	const QPointF rightPoint(right, yCenter);

	const QPointF topLeft(xCenter - quarterSide, top);
	const QPointF topRight(xCenter + quarterSide, top);
	const QPointF bottomLeft(xCenter - quarterSide, yCenter);
	const QPointF bottomRight(xCenter + quarterSide, yCenter);

	QPainterPath painterPath;

	painterPath.moveTo(bottomPoint);
	painterPath.lineTo(leftPoint);
	painterPath.lineTo(bottomLeft);
	painterPath.lineTo(topLeft);
	painterPath.lineTo(topRight);
	painterPath.lineTo(bottomRight);
	painterPath.lineTo(rightPoint);
	painterPath.closeSubpath();

	painter.setBrush(QBrush(QColor(Qt::red)));
	painter.drawPath(painterPath);
}

int main(int argc, char *argv[])
{
	QApplication app(argc, argv);

	Widget* w = new Widget();
	w->resize(500, 300);
	w->show();

	return app.exec();
}
